﻿using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace Application
{
    public class User
    {
        public String FirstName { get; set; }
        public String LastName { get; set; }
        public String City { get; set; }
        public String Pesel { get; set; }


        //checking pesel without gender (sex)
        public int CheckingPesel()
        {
            /*
             Returns:
             0 - valid pesel
             1 - invalid control digit
             2 - invalid date of birth
             3 - invalid characters of pesel
             4 - invalid length of pesel
             */
            int flag = 4;

            Regex regex = new Regex("^[0-9]*$");
           
            /// 1. Checking length of pesel  
            if (Pesel.Length == 11)
            {
                flag--;
                /// 2. Checking characters  
                if (regex.IsMatch(Pesel))
                {
                    flag--;
                    /// 3. Checking date of birth  
                    if (CheckingDateOfBirth(Pesel))
                    {
                        flag--;
                        /// 4. Checking control digit  
                        if (CheckingControlDigit(Pesel))
                        {
                            flag--;
                        }
                    }

                }
            }


            return flag;
        }

        private static bool CheckingDateOfBirth(String Pesel)
        {
            bool flag = true;
            String year = Pesel.Substring(0, 2);
            String month = Pesel.Substring(2, 2);
            String day = Pesel.Substring(4, 2);

            //checking month
            int m1 = Convert.ToInt32(month.Substring(0, 1));
            int m2 = Convert.ToInt32(month.Substring(1, 1));

            if (m1 % 2 != 0)
            {
                if (m2 > 2)
                {
                    flag = false;
                }
            }

            int m = (m1 % 2) * 10 + m2;

            int d1 = Convert.ToInt32(day.Substring(0, 1));
            int d2 = Convert.ToInt32(day.Substring(1, 1));
            int d = d1 * 10 + d2;

            int y1 = Convert.ToInt32(year.Substring(0, 1));
            int y2 = Convert.ToInt32(year.Substring(1, 1));
            int y = y1 * 10 + y2;

            if (m1 <= 1)
                y = 1900 + y;
            else if (m1 <= 3)
                y = 2000 + y;
            else if (m1 <= 5)
                y = 2100 + y;
            else if (m1 <= 7)
                y = 2200 + y;
            else
                y = 1800 + y;
            //Console.WriteLine("Date:" +d+" "+m +" "+y);

            int[] daysMonth = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

            //checking month
            if (m > 12 || m < 1)
            {
                flag = false;
            }
            /// checking days in month
            ///
            //for Februrary
            else if (m == 2)
            {
                if (!(y % 4 == 0 && y % 100 != 0 || y % 400 == 0))
                    flag = false;
            }
            //for others months
            else if (d > daysMonth[m - 1])
            {
                flag = false;
            }

            return flag;
        }

        private static bool CheckingControlDigit(String Pesel)
        {
            bool flag = false;
            int result = 0;
            int[] coff = { 9, 7, 3, 1, 9, 7, 3, 1, 9, 7 };//coefficient
            int[] digit_of_pesel = new int[11];
            int index = 0;

            foreach (var s in Pesel)
            {
                digit_of_pesel[index++] = Convert.ToInt32(s) - Convert.ToInt32('0');
            }
            for (int i = 0; i < coff.Length; i++)
            {
                result += (coff[i] * digit_of_pesel[i]);
                //Console.WriteLine(s);
            }

            result %= 10;

            if (result == digit_of_pesel[10])
                flag = true;

            return flag;

        }

        public static bool UserExists(ref List<User> list_of_users, User user)
        {
            bool flag = false;

            int index = 0;
            foreach (var u in list_of_users)
            {
                if (u.Pesel == user.Pesel
                    && (u.FirstName != user.FirstName
                    || u.LastName != user.LastName))
                {

                    flag = true;
                    list_of_users[index].FirstName = user.FirstName;
                    list_of_users[index].LastName = user.LastName;
                    list_of_users[index].City = user.City;
                    break;
                }
                else if (u.Pesel == user.Pesel
                    && u.FirstName == user.FirstName
                    && u.LastName == user.LastName)
                {
                    flag = true;
                    break;
                }
                index++;
            }

            return flag;

        }

        public override bool Equals(Object obj)
        {

            if (!(obj is User)) throw new Exception("Incorrect parameter! It should be object of User class!");
           
            User user = obj as User;

            bool condition = this.FirstName==user.FirstName && 
                             this.LastName== user.LastName && this.Pesel==user.Pesel;

            if (condition)
                return true;
            else
                return false;
        }
    }


}
